MasterLootReminder = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0","AceConsole-2.0")

local BB = AceLibrary("Babble-Boss-2.2")

MasterLootReminder.visible = false
MasterLootReminder.bossName = "_NONE_"
MasterLootReminder.Ignored = { }

MasterLootReminder:RegisterChatCommand({ "/mlr", "/masterlootreminder" }, {
    type = "group",
    args = {
        Enable = {
            name = "Enable",
            desc = "Enable/Disable 'Master Loot Reminder'",
            type = "toggle",
            get = 	function()
            			return MasterLootReminder:GetOption("Enable")
            		end,
            set =	function(v)
            			MasterLootReminder:SetOption("Enable", v)
            		end,
            order = 1,
        },
        Type = {
            name = "ML Party/Raid/Both option",
            desc = "1 = Party only, 2 = Raid only, 3 = Both",
            type = "range",
            get = 	function()
            			return MasterLootReminder:GetOption("Type")
            		end,
            set =	function(v)
            			MasterLootReminder:SetOption("Type", v)
            		end,
            min = 1,
            max = 3,
            order = 2
        },
        IgnoreTarget = {
            name = "Add target to 'permanent' ignore list",
            desc = "Add target to 'permanent' ignore list",
            type = "execute",
            func = "IgnoreTarget",
            order = 3
        },
        pIgnore = {
			type = 'group',
			name = "Permanent ignore list options",
			desc = "Permanent ignore list options",
			args = {                
		        Reset = {
		            name = "Reset permanent ignore list",
		            desc = "Reset permanent ignore list",
		            type = "execute",
		            func =	function ()
		            			MasterLootReminder:ResetIgnoreList(MasterLootReminderDB.Ignored, "Permanent")
		            		end,
		        },
		        List = {
		            name = "View permanent ignore list",
		            desc = "View permanent ignore list",
		            type = "execute",
		            func =	function ()
		            			MasterLootReminder:ViewIgnoreList(MasterLootReminderDB.Ignored, "Permanent")
		            		end,
		        },  
		        Del = {
		            name = "Delete name from permanent ignore list",
		            desc = "Delete name from permanent ignore list",
		            type = "text",
		            usage = "<name>",
				    get = false,
				    set = function(name)
				        MasterLootReminder:DelFromIgnore(name, MasterLootReminderDB.Ignored, "Permanent")
				    end,
		        },
			},
		},
        sIgnore = {
			type = 'group',
			name = "Session ignore list options",
			desc = "Session ignore list options",
			args = {                
		        Reset = {
		            name = "Reset ignore list",
		            desc = "Reset ignore list",
		            type = "execute",
		            func =	function ()
		            			MasterLootReminder:ResetIgnoreList(MasterLootReminder.Ignored, "Session")
		            		end,
		        },
		        List = {
		            name = "View ignore list",
		            desc = "View ignore list",
		            type = "execute",
		            func =	function ()
		            			MasterLootReminder:ViewIgnoreList(MasterLootReminder.Ignored, "Session")
		            		end,
		        },  
		        Del = {
		            name = "Delete name from ignore list",
		            desc = "Delete name from ignore list",
		            type = "text",
		            usage = "<name>",
				    get = false,
				    set = function(name)
				        MasterLootReminder:DelFromIgnore(name, MasterLootReminder.Ignored, "Session")
				    end,
		        },
			},
		},
	},
})	

function MasterLootReminder:GetOption(key)
	return MasterLootReminderDB[key]
end

function MasterLootReminder:SetOption(key, value)
	MasterLootReminderDB[key] = value
end

function MasterLootReminder:IgnoreTarget()
	local targetName = UnitName("target")
	if targetName == nil then
		MasterLootReminder:Print("No target found.")
		return
	end
	if not BB:HasTranslation(targetName) then
		MasterLootReminder:Print(targetName .. " is not a known boss.")
		return		
	end
	if MasterLootReminder:inTable(MasterLootReminderDB.Ignored,targetName) then
		MasterLootReminder:Print(targetName .. " is already being ignored.")
		return
	end
	table.insert(MasterLootReminderDB.Ignored,targetName)
	MasterLootReminder:Print(targetName .. " now permanently ignored!")
end

function MasterLootReminder:ResetIgnoreList(tableName, typeoflist)
	table.remove(tableName)
	MasterLootReminder:Print(typeoflist .. " ignore list reset!")
end

function MasterLootReminder:ViewIgnoreList(tableName, typeoflist)
	if #tableName == 0 then
		self:Print(typeoflist .. " ignore list is empty!")
		return
	end
	self:Print(typeoflist .. " ignore list:")
	for index, value in pairs(tableName) do
		self:Print(value)
	end
end

function MasterLootReminder:DelFromIgnore(name, tableName, typeoflist)
	if #tableName == 0 then
		self:Print(typeoflist .. " ignore list is empty!")
		return
	end
	local i
	i = MasterLootReminder:inTable(tableName,name)
	if i ~= nil then
		table.remove(tableName,i)
		self:Print(name .. " removed from " .. typeoflist .. " ignore list!")	
	else
		self:Print(name .. " not found in " .. typeoflist .. " ignore list!")	
	end
end

function MasterLootReminder:OnEnable()
	if MasterLootReminderDB == nil then
		MasterLootReminderDB = { }
		MasterLootReminderDB.Enable = true
		MasterLootReminderDB.Type = 3
		MasterLootReminderDB.Ignored = { }
	end
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end

StaticPopupDialogs["MASTERLOOTREMINDER_POPUP"] = {
	text     = "Set yourself as Master Looter?",
	button1  = TEXT(YES),
	button2  = TEXT(NO),
	OnShow   = function()
		MasterLootReminder.visible = true
	end,
	OnAccept = function()
		SetLootMethod("master",UnitName("player"))		
		MasterLootReminder.visible = false
	end,
	OnCancel = function()
		MasterLootReminder:Ignore(MasterLootReminder.Ignored)
		MasterLootReminder.visible = false
	end,
	timeout = 0,
	whileDead = 1
}

function MasterLootReminder:PLAYER_TARGET_CHANGED()
	local type = MasterLootReminderDB.Type
	local getType = MasterLootReminder:GetType()
	if MasterLootReminderDB.Enable and (getType ~= 0 and type == 3 or getType == type) then
		local lootmethod = GetLootMethod()
		local targetName
		if UnitIsPlayer("target") or UnitPlayerControlled("target") then
			targetName = UnitName("targettarget")
		else
			targetName = UnitName("target")
		end
		if targetName ~= nil and lootmethod ~= "master" and MasterLootReminder.visible == false and BB:HasTranslation(targetName) then 
			if MasterLootReminder:isIgnored(targetName) == false then
				MasterLootReminder.bossName = targetName
				MasterLootReminder:Print(MasterLootReminder.bossName)
				StaticPopup_Show("MASTERLOOTREMINDER_POPUP")
			end
		end
	end
end

function MasterLootReminder:GetType()
	-- 1 = Party only, 2 = Raid only, 3 = Both (2 is really both)
	if not IsPartyLeader() then
		return 0
	elseif UnitInParty("player") and not UnitInRaid("player") then
		return 1
	elseif UnitInParty("player") and UnitInRaid("player") then
		return 2
	end
end

function MasterLootReminder:isIgnored(UnitName)
	if MasterLootReminder:inTable(MasterLootReminderDB.Ignored,UnitName) or MasterLootReminder:inTable(MasterLootReminder.Ignored,UnitName) then
		return true
	else
		return false
	end
end

function MasterLootReminder:Ignore(tableName)
	if not UnitName then return end
	if MasterLootReminder:inTable(tableName,MasterLootReminder.bossName) ~= nil then
		return
	end
	MasterLootReminder:Print("Now ignoring: "..MasterLootReminder.bossName)
	table.insert(tableName,MasterLootReminder.bossName)
end

function MasterLootReminder:inTable(tableName,searchString)
	if not searchString then return false end
	for index, value in pairs(tableName) do
		if string.lower(value) == string.lower(searchString) then
			return index
		end
	end
	return nil
end